// src/esl.litcoffee
import net from "node:net";
import EventEmitter2, {
  once
} from "node:events";

// src/response.litcoffee
import EventEmitter from "node:events";

// node_modules/ulidx/dist/node/index.js
import crypto from "node:crypto";

// node_modules/layerr/dist/error.js
function assertError(err) {
  if (!isError(err)) {
    throw new Error("Parameter was not an error");
  }
}
function isError(err) {
  return objectToString(err) === "[object Error]" || err instanceof Error;
}
function objectToString(obj) {
  return Object.prototype.toString.call(obj);
}

// node_modules/layerr/dist/tools.js
function parseArguments(args) {
  let options, shortMessage = "";
  if (args.length === 0) {
    options = {};
  } else if (isError(args[0])) {
    options = {
      cause: args[0]
    };
    shortMessage = args.slice(1).join(" ") || "";
  } else if (args[0] && typeof args[0] === "object") {
    options = Object.assign({}, args[0]);
    shortMessage = args.slice(1).join(" ") || "";
  } else if (typeof args[0] === "string") {
    options = {};
    shortMessage = shortMessage = args.join(" ") || "";
  } else {
    throw new Error("Invalid arguments passed to Layerr");
  }
  return {
    options,
    shortMessage
  };
}

// node_modules/layerr/dist/layerr.js
var Layerr = class _Layerr extends Error {
  constructor(errorOptionsOrMessage, messageText) {
    const args = [...arguments];
    const { options, shortMessage } = parseArguments(args);
    let message = shortMessage;
    if (options.cause) {
      message = `${message}: ${options.cause.message}`;
    }
    super(message);
    this.message = message;
    if (options.name && typeof options.name === "string") {
      this.name = options.name;
    } else {
      this.name = "Layerr";
    }
    if (options.cause) {
      Object.defineProperty(this, "_cause", { value: options.cause });
    }
    Object.defineProperty(this, "_info", { value: {} });
    if (options.info && typeof options.info === "object") {
      Object.assign(this._info, options.info);
    }
    if (Error.captureStackTrace) {
      const ctor = options.constructorOpt || this.constructor;
      Error.captureStackTrace(this, ctor);
    }
  }
  static cause(err) {
    assertError(err);
    if (!err._cause)
      return null;
    return isError(err._cause) ? err._cause : null;
  }
  static fullStack(err) {
    assertError(err);
    const cause = _Layerr.cause(err);
    if (cause) {
      return `${err.stack}
caused by: ${_Layerr.fullStack(cause)}`;
    }
    return err.stack;
  }
  static info(err) {
    assertError(err);
    const output = {};
    const cause = _Layerr.cause(err);
    if (cause) {
      Object.assign(output, _Layerr.info(cause));
    }
    if (err._info) {
      Object.assign(output, err._info);
    }
    return output;
  }
  cause() {
    return _Layerr.cause(this);
  }
  toString() {
    let output = this.name || this.constructor.name || this.constructor.prototype.name;
    if (this.message) {
      output = `${output}: ${this.message}`;
    }
    return output;
  }
};

// node_modules/ulidx/dist/node/index.js
var ENCODING = "0123456789ABCDEFGHJKMNPQRSTVWXYZ";
var ENCODING_LEN = 32;
var TIME_MAX = 281474976710655;
var TIME_LEN = 10;
var RANDOM_LEN = 16;
var ERROR_INFO = Object.freeze({
  source: "ulid"
});
function detectPRNG(root) {
  const rootLookup = root || detectRoot();
  const globalCrypto = rootLookup && (rootLookup.crypto || rootLookup.msCrypto) || (typeof crypto !== "undefined" ? crypto : null);
  if (typeof globalCrypto?.getRandomValues === "function") {
    return () => {
      const buffer = new Uint8Array(1);
      globalCrypto.getRandomValues(buffer);
      return buffer[0] / 255;
    };
  } else if (typeof globalCrypto?.randomBytes === "function") {
    return () => globalCrypto.randomBytes(1).readUInt8() / 255;
  } else if (crypto?.randomBytes) {
    return () => crypto.randomBytes(1).readUInt8() / 255;
  }
  throw new Layerr({
    info: {
      code: "PRNG_DETECT",
      ...ERROR_INFO
    }
  }, "Failed to find a reliable PRNG");
}
function detectRoot() {
  if (inWebWorker())
    return self;
  if (typeof window !== "undefined") {
    return window;
  }
  if (typeof global !== "undefined") {
    return global;
  }
  if (typeof globalThis !== "undefined") {
    return globalThis;
  }
  return null;
}
function encodeRandom(len, prng) {
  let str = "";
  for (; len > 0; len--) {
    str = randomChar(prng) + str;
  }
  return str;
}
function encodeTime(now, len) {
  if (isNaN(now)) {
    throw new Layerr({
      info: {
        code: "ENC_TIME_NAN",
        ...ERROR_INFO
      }
    }, `Time must be a number: ${now}`);
  } else if (now > TIME_MAX) {
    throw new Layerr({
      info: {
        code: "ENC_TIME_SIZE_EXCEED",
        ...ERROR_INFO
      }
    }, `Cannot encode a time larger than ${TIME_MAX}: ${now}`);
  } else if (now < 0) {
    throw new Layerr({
      info: {
        code: "ENC_TIME_NEG",
        ...ERROR_INFO
      }
    }, `Time must be positive: ${now}`);
  } else if (Number.isInteger(now) === false) {
    throw new Layerr({
      info: {
        code: "ENC_TIME_TYPE",
        ...ERROR_INFO
      }
    }, `Time must be an integer: ${now}`);
  }
  let mod, str = "";
  for (let currentLen = len; currentLen > 0; currentLen--) {
    mod = now % ENCODING_LEN;
    str = ENCODING.charAt(mod) + str;
    now = (now - mod) / ENCODING_LEN;
  }
  return str;
}
function inWebWorker() {
  return typeof WorkerGlobalScope !== "undefined" && self instanceof WorkerGlobalScope;
}
function randomChar(prng) {
  let rand = Math.floor(prng() * ENCODING_LEN);
  if (rand === ENCODING_LEN) {
    rand = ENCODING_LEN - 1;
  }
  return ENCODING.charAt(rand);
}
function ulid(seedTime, prng) {
  const currentPRNG = prng || detectPRNG();
  const seed = isNaN(seedTime) ? Date.now() : seedTime;
  return encodeTime(seed, TIME_LEN) + encodeRandom(RANDOM_LEN, currentPRNG);
}

// src/parser.litcoffee
import querystring from "node:querystring";
var FreeSwitchParserError = class FreeSwitchParserError2 extends Error {
  constructor(error, buffer) {
    super(JSON.stringify({ error, buffer }));
    this.error = error;
    this.buffer = buffer;
    return;
  }
};
var FreeSwitchParser = class FreeSwitchParser2 {
  // The Event Socket parser will parse an incoming ES stream, whether your code is acting as a client (connected to the FreeSwitch ES server) or as a server (called back by FreeSwitch due to the "socket" application command).
  constructor(socket, process) {
    this.process = process;
    this.body_length = 0;
    this.buffer = Buffer.alloc(0);
    this.buffer_length = 0;
    socket.on("data", (data) => {
      if (this.body_length > 0) {
        this.capture_body(data);
      } else {
        this.capture_headers(data);
      }
    });
    socket.once("end", () => {
      if (this.buffer_length > 0) {
        socket.emit("warning", new FreeSwitchParserError("Buffer is not empty at end of stream", this.buffer));
      }
    });
    return;
  }
  // ### Capture body
  capture_body(data) {
    var body;
    this.buffer_length += data.length;
    this.buffer = Buffer.concat([this.buffer, data], this.buffer_length);
    if (this.buffer_length < this.body_length) {
      return;
    }
    body = this.buffer.toString("utf8", 0, this.body_length);
    this.buffer = this.buffer.slice(this.body_length);
    this.buffer_length -= this.body_length;
    this.body_length = 0;
    this.process(this.headers, body);
    this.headers = {};
    this.capture_headers(Buffer.alloc(0));
  }
  // ### Capture headers
  capture_headers(data) {
    var header_end, header_text;
    this.buffer_length += data.length;
    this.buffer = Buffer.concat([this.buffer, data], this.buffer_length);
    header_end = this.buffer.indexOf("\n\n");
    if (header_end < 0) {
      return;
    }
    header_text = this.buffer.toString("utf8", 0, header_end);
    this.buffer = this.buffer.slice(header_end + 2);
    this.buffer_length -= header_end + 2;
    this.headers = parse_header_text(header_text);
    if (this.headers["Content-Length"]) {
      this.body_length = parseInt(this.headers["Content-Length"], 10);
      this.capture_body(Buffer.alloc(0));
    } else {
      this.process(this.headers);
      this.headers = {};
      this.capture_headers(Buffer.alloc(0));
    }
  }
};
var parse_header_text = function(header_text) {
  var header_lines, headers, i, len, line, name, ref, ref1;
  header_lines = header_text.split("\n");
  headers = {};
  for (i = 0, len = header_lines.length; i < len; i++) {
    line = header_lines[i];
    (function(line2) {
      var name2, value;
      [name2, value] = line2.split(/: /, 2);
      return headers[name2] = value;
    })(line);
  }
  if (((ref = headers["Reply-Text"]) != null ? ref[0] : void 0) === "%") {
    for (name in headers) {
      headers[name] = querystring.unescape((ref1 = headers[name]) != null ? ref1 : "");
    }
  }
  return headers;
};

// src/response.litcoffee
import assert from "node:assert";
var FreeSwitchError;
var FreeSwitchTimeout;
var async_log;
async_log = function(msg, af, logger) {
  return function() {
    return af().catch(function(error) {
      logger.error(`FreeSwitchResponse::async_log: ${msg}`, { error });
      throw error;
    });
  };
};
FreeSwitchError = class FreeSwitchError2 extends Error {
  constructor(res, args) {
    super();
    this.res = res;
    this.args = args;
    return;
  }
  toString() {
    return `FreeSwitchError: ${JSON.stringify(this.args)}`;
  }
};
FreeSwitchTimeout = class FreeSwitchTimeout2 extends Error {
  constructor(timeout, text) {
    super();
    this.timeout = timeout;
    this.text = text;
    return;
  }
  toString() {
    return `FreeSwitchTimeout: Timeout after ${this.timeout}ms waiting for ${this.text}`;
  }
};
var FreeSwitchResponse = function() {
  class FreeSwitchResponse2 extends EventEmitter {
    // The `FreeSwitchResponse` is bound to a single socket (dual-stream). For outbound (server) mode this would represent a single socket call from FreeSwitch.
    constructor(socket, logger) {
      var once_socket_star, socket_on_error, socket_once_close;
      super({
        captureRejections: true
      });
      this.setMaxListeners(2e3);
      socket.setKeepAlive(true);
      socket.setNoDelay(true);
      assert(socket != null, "Missing socket parameter");
      assert.equal("function", typeof socket.once, "FreeSwitchResponse: socket.once must be a function");
      assert.equal("function", typeof socket.on, "FreeSwitchResponse: socket.on must be a function");
      assert.equal("function", typeof socket.end, "FreeSwitchResponse: socket.end must be a function");
      assert.equal("function", typeof socket.write, "FreeSwitchResponse: socket.write must be a function");
      assert(logger != null, "Missing logger parameter");
      assert.equal("function", typeof logger.debug, "FreeSwitchResponse: logger.debug must be a function");
      assert.equal("function", typeof logger.info, "FreeSwitchResponse: logger.info must be a function");
      assert.equal("function", typeof logger.error, "FreeSwitchResponse: logger.error must be a function");
      this.__ref = ulid();
      this.__uuid = null;
      this.__socket = socket;
      this.logger = logger;
      this.stats = {
        missing_content_type: 0n,
        auth_request: 0n,
        command_reply: 0n,
        events: 0n,
        json_parse_errors: 0n,
        log_data: 0n,
        disconnect: 0n,
        api_responses: 0n,
        rude_rejections: 0n,
        unhandled: 0n
      };
      this.on("CHANNEL_EXECUTE_COMPLETE", (res) => {
        var event_uuid;
        event_uuid = res.body["Application-UUID"];
        this.logger.debug("FreeSwitchResponse: CHANNEL_EXECUTE_COMPLETE", { event_uuid });
        this.emit(`CHANNEL_EXECUTE_COMPLETE ${event_uuid}`, res);
      });
      this.on("BACKGROUND_JOB", (res) => {
        var job_uuid;
        job_uuid = res.body["Job-UUID"];
        this.logger.debug("FreeSwitchResponse: BACKGROUND_JOB", { job_uuid });
        this.emit_later(`BACKGROUND_JOB ${job_uuid}`, {
          body: res.body._body
        });
      });
      this.__parser = new FreeSwitchParser(this.__socket, (headers, body) => {
        return this.process(headers, body);
      });
      this.__queue = Promise.resolve(null);
      this.__later = /* @__PURE__ */ new Map();
      this.closed = false;
      socket_once_close = () => {
        this.logger.debug("FreeSwitchResponse: Socket closed", {
          ref: this.__ref
        });
        this.emit("socket.close");
      };
      this.__socket.once("close", socket_once_close);
      socket_on_error = (err) => {
        this.logger.debug("FreeSwitchResponse: Socket Error", {
          ref: this.__ref,
          error: err
        });
        this.emit("socket.error", err);
      };
      this.__socket.on("error", socket_on_error);
      once_socket_star = (reason) => {
        this.logger.debug("FreeSwitchResponse: Terminate", {
          ref: this.__ref,
          reason
        });
        if (!this.closed) {
          this.closed = true;
          this.__socket.end();
        }
        this.removeAllListeners();
        this.__queue = Promise.resolve(null);
        this.__later.clear();
      };
      this.once("socket.error", once_socket_star);
      this.once("socket.close", once_socket_star);
      this.once("socket.write", once_socket_star);
      this.once("socket.end", once_socket_star);
      null;
    }
    setUUID(uuid) {
      this.__uuid = uuid;
    }
    uuid() {
      return this.__uuid;
    }
    ref() {
      return this.__ref;
    }
    error(res, data) {
      this.logger.error("FreeSwitchResponse: error: new FreeSwitchError", {
        ref: this.__ref,
        res,
        data
      });
      throw new FreeSwitchError(res, data);
    }
    // onceAsync
    // ---------
    /**
     * @param {string} event
     * @param {number} timeout
     * @param {string} comment
     */
    onceAsync(event, timeout, comment) {
      var onceAsyncHandler;
      this.logger.debug("FreeSwitchResponse: onceAsync: awaiting", {
        event,
        comment,
        ref: this.__ref,
        timeout
      });
      onceAsyncHandler = (resolve, reject) => {
        var cleanup, on_close, on_end, on_error, on_event, on_timeout, timer;
        on_event = (...args) => {
          this.logger.debug("FreeSwitchResponse: onceAsync: on_event", {
            event,
            comment,
            ref: this.__ref
          });
          cleanup();
          resolve(...args);
        };
        on_error = (error) => {
          this.logger.error("FreeSwitchResponse: onceAsync: on_error", {
            event,
            comment,
            ref: this.__ref,
            error
          });
          cleanup();
          reject(error);
        };
        on_close = () => {
          this.logger.error("FreeSwitchResponse: onceAsync: on_close", {
            event,
            comment,
            ref: this.__ref
          });
          cleanup();
          reject(new Error(`Socket closed (${this.__ref}) while waiting for ${event} in ${comment}`));
        };
        on_end = () => {
          this.logger.error("FreeSwitchResponse: onceAsync: on_end", {
            event,
            comment,
            ref: this.__ref
          });
          cleanup();
          reject(new Error(`end() called (${this.__ref}) while waiting for ${event} in ${comment}`));
        };
        on_timeout = () => {
          this.logger.error("FreeSwitchResponse: onceAsync: on_timeout", {
            event,
            comment,
            ref: this.__ref,
            timeout
          });
          cleanup();
          reject(new FreeSwitchTimeout(timeout, `(${this.__ref}) event ${event} in ${comment}`));
        };
        cleanup = () => {
          this.removeListener(event, on_event);
          this.removeListener("socket.error", on_error);
          this.removeListener("socket.close", on_close);
          this.removeListener("socket.write", on_error);
          this.removeListener("socket.end", on_end);
          clearTimeout(timer);
        };
        this.once(event, on_event);
        this.once("socket.error", on_error);
        this.once("socket.close", on_close);
        this.once("socket.write", on_error);
        this.once("socket.end", on_end);
        if (timeout != null) {
          timer = setTimeout(on_timeout, timeout);
        }
      };
      return new Promise(onceAsyncHandler);
    }
    // Queueing
    // ========
    // Enqueue a function that returns a Promise.
    // The function is only called when all previously enqueued functions-that-return-Promises are completed and their respective Promises fulfilled or rejected.
    enqueue(f) {
      var next, q;
      if (this.closed) {
        return this.error({}, {
          when: "enqueue on closed socket"
        });
      }
      q = this.__queue;
      next = async function() {
        await q;
        return await f();
      }();
      this.__queue = next.catch(function() {
        return true;
      });
      return next;
    }
    // Sync/Async event
    // ================
    // waitAsync
    // ---------
    // In some cases the event might have been emitted before we are ready to receive it.
    // In that case we store the data in `@__later` so that we can emit the event when the recipient is ready.
    waitAsync(event, timeout, comment) {
      var v;
      if (!this.closed && this.__later.has(event)) {
        v = this.__later.get(event);
        this.__later.delete(event);
        return Promise.resolve(v);
      } else {
        return this.onceAsync(event, timeout, `waitAsync ${comment}`);
      }
    }
    // emit_later
    // ----------
    // This is used for events that might trigger before we set the `once` receiver.
    emit_later(event, data) {
      var handled;
      handled = this.emit(event, data);
      if (!this.closed && !handled) {
        this.__later.set(event, data);
      }
      return handled;
    }
    // Low-level sending
    // =================
    // These methods are normally not used directly.
    // write
    // -----
    // Send a single command to FreeSwitch; `args` is a hash of headers sent with the command.
    write(command, args) {
      var writeHandler;
      if (this.closed) {
        return this.error({}, {
          when: "write on closed socket",
          command,
          args
        });
      }
      writeHandler = (resolve, reject) => {
        var error, flushed, key, text, value;
        try {
          this.logger.debug("FreeSwitchResponse: write", {
            ref: this.__ref,
            command,
            args
          });
          text = `${command}
`;
          if (args != null) {
            for (key in args) {
              value = args[key];
              if (value != null) {
                text += `${key}: ${value}
`;
              }
            }
          }
          text += "\n";
          this.logger.debug("FreeSwitchResponse: write", {
            ref: this.__ref,
            text
          });
          flushed = this.__socket.write(text, "utf8");
          if (!flushed) {
            this.logger.debug("FreeSwitchResponse: write did not flush");
          }
          resolve(null);
        } catch (error1) {
          error = error1;
          this.logger.error("FreeSwitchResponse: write error", {
            ref: this.__ref,
            error
          });
          this.emit("socket.write", error);
          reject(error);
        }
      };
      return new Promise(writeHandler);
    }
    // send
    // ----
    // A generic way of sending commands to FreeSwitch, wrapping `write` into a Promise that waits for FreeSwitch's notification that the command completed.
    /**
     * @param {string} command
     * @param {object?} args
     * @param {number} timeout
     */
    async send(command, args = void 0, timeout = this.default_send_timeout) {
      var msg, sendHandler;
      if (this.closed) {
        return this.error({}, {
          when: "send on closed socket",
          command,
          args
        });
      }
      msg = `send ${command} ${JSON.stringify(args)}`;
      sendHandler = async () => {
        var p, q, reply, res;
        p = this.onceAsync("freeswitch_command_reply", timeout, msg);
        q = this.write(command, args);
        [res] = await Promise.all([p, q]);
        this.logger.debug("FreeSwitchResponse: send: received reply", {
          ref: this.__ref,
          command,
          args
        });
        reply = res != null ? res.headers["Reply-Text"] : void 0;
        if (reply == null) {
          this.logger.debug("FreeSwitchResponse: send: no reply", {
            ref: this.__ref,
            command,
            args
          });
          return this.error(res, {
            when: "no reply to command",
            command,
            args
          });
        }
        if (reply.match(/^-/)) {
          this.logger.debug("FreeSwitchResponse: send: failed", { __ref: this.__ref, reply, command, args });
          return this.error(res, {
            when: "command reply",
            reply,
            command,
            args
          });
        }
        this.logger.debug("FreeSwitchResponse: send: success", {
          ref: this.__ref,
          command,
          args
        });
        return res;
      };
      return await this.enqueue(async_log(msg, sendHandler, this.logger));
    }
    // end
    // ---
    // Closes the socket.
    end() {
      this.logger.debug("FreeSwitchResponse: end", {
        ref: this.__ref
      });
      this.emit("socket.end", "Socket close requested by application");
    }
    // Process data from the parser
    // ============================
    // Rewrite headers as needed to work around some weirdnesses in the protocol; and assign unified event IDs to the Event Socket's Content-Types.
    process(headers, body) {
      var content_type, event, exception, i, len, msg, n, ref;
      this.logger.debug("FreeSwitchResponse::process", {
        ref: this.__ref,
        headers,
        body
      });
      content_type = headers["Content-Type"];
      if (content_type == null) {
        this.stats.missing_content_type++;
        this.logger.error("FreeSwitchResponse::process: missing-content-type", {
          ref: this.__ref,
          headers,
          body
        });
        this.emit("error.missing-content-type", new FreeSwitchParserError({ headers, body }));
        return;
      }
      switch (content_type) {
        case "auth/request":
          event = "freeswitch_auth_request";
          this.stats.auth_request++;
          break;
        case "command/reply":
          event = "freeswitch_command_reply";
          if (headers["Event-Name"] === "CHANNEL_DATA") {
            body = headers;
            headers = {};
            ref = ["Content-Type", "Reply-Text", "Socket-Mode", "Control"];
            for (i = 0, len = ref.length; i < len; i++) {
              n = ref[i];
              headers[n] = body[n];
              delete body[n];
            }
          }
          this.stats.command_reply++;
          break;
        case "text/event-json":
          this.stats.events++;
          try {
            body = body.replace(/[\x00-\x1F\x7F-\x9F]/g, "");
            body = JSON.parse(body);
          } catch (error1) {
            exception = error1;
            this.logger.error("FreeSwitchResponse: Invalid JSON", {
              ref: this.__ref,
              body
            });
            this.stats.json_parse_errors++;
            this.emit("error.invalid-json", exception);
            return;
          }
          event = body["Event-Name"];
          break;
        case "text/event-plain":
          body = parse_header_text(body);
          event = body["Event-Name"];
          this.stats.events++;
          break;
        case "log/data":
          event = "freeswitch_log_data";
          this.stats.log_data++;
          break;
        case "text/disconnect-notice":
          event = "freeswitch_disconnect_notice";
          this.stats.disconnect++;
          break;
        case "api/response":
          event = "freeswitch_api_response";
          this.stats.api_responses++;
          break;
        case "text/rude-rejection":
          event = "freeswitch_rude_rejection";
          this.stats.rude_rejections++;
          break;
        default:
          this.logger.error("FreeSwitchResponse: Unhandled Content-Type", {
            ref: this.__ref,
            content_type
          });
          event = `freeswitch_${content_type.replace(/[^a-z]/, "_")}`;
          this.emit("error.unhandled-content-type", new FreeSwitchParserError({ content_type }));
          this.stats.unhandled++;
      }
      msg = { headers, body };
      this.emit(event, msg);
    }
    // Channel-level commands
    // ======================
    // api
    // ---
    // Send an API command, see [Mod commands](http://wiki.freeswitch.org/wiki/Mod_commands) for a list.
    // Returns a Promise that is fulfilled as soon as FreeSwitch sends a reply. Requests are queued and each request is matched with the first-coming response, since there is no way to match between requests and responses.
    // Use `bgapi` if you need to make sure responses are correct, since it provides the proper semantices.
    async api(command, timeout) {
      var apiHandler, msg;
      this.logger.debug("FreeSwitchResponse: api", {
        ref: this.__ref,
        command
      });
      if (this.closed) {
        return this.error({}, {
          when: "api on closed socket",
          command
        });
      }
      msg = `api ${command}`;
      apiHandler = async () => {
        var p, q, ref, reply, res;
        p = this.onceAsync("freeswitch_api_response", timeout, msg);
        q = this.write(`api ${command}`);
        [res] = await Promise.all([p, q]);
        this.logger.debug("FreeSwitchResponse: api: response", {
          ref: this.__ref,
          command
        });
        reply = res != null ? res.body : void 0;
        if (reply == null) {
          this.logger.debug("FreeSwitchResponse: api: no reply", { __ref: this.__ref, command });
          return this.error(res, {
            when: "no reply to api",
            command
          });
        }
        if (reply.match(/^-/)) {
          this.logger.debug("FreeSwitchResponse: api response failed", {
            ref: this.__ref,
            reply,
            command
          });
          return this.error(res, {
            when: "api response",
            reply,
            command
          });
        }
        res.uuid = (ref = reply.match(/^\+OK (\S+)/)) != null ? ref[1] : void 0;
        return res;
      };
      return await this.enqueue(async_log(msg, apiHandler, this.logger));
    }
    // bgapi
    // -----
    // Send an API command in the background. Wraps it inside a Promise.
    async bgapi(command, timeout) {
      var error, r, ref, reply, res;
      this.logger.debug("FreeSwitchResponse: bgapi", {
        ref: this.__ref,
        command,
        timeout
      });
      if (this.closed) {
        return this.error({}, {
          when: "bgapi on closed socket",
          command
        });
      }
      res = await this.send(`bgapi ${command}`);
      error = () => {
        return this.error(res, {
          when: "bgapi did not provide a Job-UUID",
          command
        });
      };
      if (res == null) {
        return error();
      }
      reply = res.headers["Reply-Text"];
      r = reply != null ? (ref = reply.match(/\+OK Job-UUID: (.+)$/)) != null ? ref[1] : void 0 : void 0;
      if (r == null) {
        r = res.headers["Job-UUID"];
      }
      if (r == null) {
        return error();
      }
      this.logger.debug("FreeSwitchResponse: bgapi retrieve", {
        ref: this.__ref,
        reply_match: r
      });
      return await this.waitAsync(`BACKGROUND_JOB ${r}`, timeout, `bgapi ${command}`);
    }
    // Event reception and filtering
    // =============================
    // event_json
    // ----------
    // Request that the server send us events in JSON format.
    // For example: `res.event_json 'HEARTBEAT'`
    event_json(...events) {
      return this.send(`event json ${events.join(" ")}`);
    }
    // nixevents
    // ---------
    // Remove the given event types from the events ACL.
    nixevent(...events) {
      return this.send(`nixevent ${events.join(" ")}`);
    }
    // noevents
    // --------
    // Remove all events types.
    noevents() {
      return this.send("noevents");
    }
    // filter
    // ------
    // Generic event filtering
    filter(header, value) {
      return this.send(`filter ${header} ${value}`);
    }
    // filter_delete
    // -------------
    // Remove a filter.
    filter_delete(header, value) {
      if (value != null) {
        return this.send(`filter delete ${header} ${value}`);
      } else {
        return this.send(`filter delete ${header}`);
      }
    }
    // sendevent
    // ---------
    // Send an event into the FreeSwitch event queue.
    sendevent(event_name, args) {
      return this.send(`sendevent ${event_name}`, args);
    }
    // Connection handling
    // ===================
    // auth
    // ----
    // Authenticate with FreeSwitch.
    // This normally not needed since in outbound (server) mode authentication is not required, and for inbound (client) mode the module authenticates automatically when requested.
    auth(password) {
      return this.send(`auth ${password}`);
    }
    // connect
    // -------
    // Used in server mode to start the conversation with FreeSwitch.
    // Normally not needed, triggered automatically by the module.
    connect() {
      return this.send("connect");
    }
    // linger
    // ------
    // Used in server mode, requests FreeSwitch to not close the socket as soon as the call is over, allowing us to do some post-processing on the call (mainly, receiving call termination events).
    // By default, `esl` with call `exit()` for you after 4 seconds. You need to capture the `cleanup_linger` event if you want to handle things differently.
    linger() {
      return this.send("linger");
    }
    // exit
    // ----
    // Send the `exit` command to the FreeSwitch socket.
    // FreeSwitch will respond with "+OK bye" followed by a `disconnect-notice` message, which gets translated into a `freeswitch_disconnect_notice` event internally, which in turn gets translated into either `freeswitch_disconnect` or `freeswitch_linger` depending on whether `linger` was called on the socket.
    // You normally do not need to call `@exit` directly. If you do, make sure you do handle any rejection.
    exit() {
      return this.send("exit");
    }
    // Event logging
    // =============
    // log
    // ---
    // Enable logging on the socket, optionnally setting the log level.
    log(level) {
      if (level != null) {
        return this.send(`log ${level}`);
      } else {
        return this.send("log");
      }
    }
    // nolog
    // -----
    // Disable logging on the socket.
    nolog() {
      return this.send("nolog");
    }
    // Message sending
    // ===============
    // sendmsg_uuid
    // ------------
    // Send a command to a given UUID.
    sendmsg_uuid(uuid, command, args) {
      var execute_text, options, res;
      options = args != null ? args : {};
      options["call-command"] = command;
      execute_text = "sendmsg";
      if (uuid != null) {
        execute_text = `sendmsg ${uuid}`;
      } else if (this.__uuid != null) {
        execute_text = `sendmsg ${this.__uuid}`;
      }
      res = this.send(execute_text, options);
      this.logger.debug("FreeSwitchResponse: sendmsg_uuid", {
        ref: this.__ref,
        uuid,
        command,
        args,
        res
      });
      return res;
    }
    // sendmsg
    // -------
    // Send Message, assuming server/outbound ESL mode (in which case the UUID is not required).
    sendmsg(command, args) {
      return this.sendmsg_uuid(null, command, args);
    }
    // Client-mode ("inbound") commands
    // =================================
    // The target UUID must be specified.
    // execute_uuid
    // ------------
    // Execute an application for the given UUID (in client mode).
    execute_uuid(uuid, app_name, app_arg, loops, event_uuid) {
      var options, res;
      options = {
        "execute-app-name": app_name,
        "execute-app-arg": app_arg,
        loops: loops != null ? loops : void 0,
        "Event-UUID": event_uuid != null ? event_uuid : void 0
      };
      res = this.sendmsg_uuid(uuid, "execute", options);
      this.logger.debug("FreeSwitchResponse: execute_uuid", {
        ref: this.__ref,
        uuid,
        app_name,
        app_arg,
        loops,
        event_uuid,
        res
      });
      return res;
    }
    // TODO: Support the alternate format (with no `execute-app-arg` header but instead a `text/plain` body containing the argument).
    // command_uuid
    // ------------
    // Execute an application synchronously. Return a Promise.
    async command_uuid(uuid, app_name, app_arg, timeout = this.default_command_timeout) {
      var event, event_uuid, p, q, res;
      if (app_arg == null) {
        app_arg = "";
      }
      event_uuid = ulid();
      event = `CHANNEL_EXECUTE_COMPLETE ${event_uuid}`;
      p = this.onceAsync(event, timeout, `uuid ${uuid} ${app_name} ${app_arg}`);
      q = this.execute_uuid(uuid, app_name, app_arg, null, event_uuid);
      [res] = await Promise.all([p, q]);
      this.logger.debug("FreeSwitchResponse: command_uuid", {
        ref: this.__ref,
        uuid,
        app_name,
        app_arg,
        timeout,
        event_uuid,
        res
      });
      return res;
    }
    // hangup_uuid
    // -----------
    // Hangup the call referenced by the given UUID with an optional (FreeSwitch) cause code.
    hangup_uuid(uuid, hangup_cause) {
      var options;
      if (hangup_cause == null) {
        hangup_cause = "NORMAL_UNSPECIFIED";
      }
      options = {
        "hangup-cause": hangup_cause
      };
      return this.sendmsg_uuid(uuid, "hangup", options);
    }
    // unicast_uuid
    // ------------
    // Forwards the media to and from a given socket.
    // Arguments:
    // - `local-ip`
    // - `local-port`
    // - `remote-ip`
    // - `remote-port`
    // - `transport` (`tcp` or `udp`)
    // - `flags: "native"` (optional: do not transcode to/from L16 audio)
    unicast_uuid(uuid, args) {
      return this.sendmsg_uuid(uuid, "unicast", args);
    }
    // nomedia_uuid
    // ------------
    // Not implemented yet (TODO).
    // Server-mode commands
    // ====================
    // In server (outbound) mode, the target UUID is always our (own) call UUID, so it does not need to be specified.
    // execute
    // -------
    // Execute an application for the current UUID (in server/outbound mode)
    execute(app_name, app_arg) {
      return this.execute_uuid(null, app_name, app_arg);
    }
    // command
    // -------
    command(app_name, app_arg) {
      return this.command_uuid(null, app_name, app_arg);
    }
    // hangup
    // ------
    hangup(hangup_cause) {
      return this.hangup_uuid(null, hangup_cause);
    }
    // unicast
    // -------
    unicast(args) {
      return this.unicast_uuid(null, args);
    }
    // TODO: `nomedia`
    // Cleanup at end of call
    // ======================
    // auto_cleanup
    // ------------
    // Clean-up at the end of the connection.
    // Automatically called by the client and server.
    auto_cleanup() {
      var linger_delay, once_freeswitch_disconnect, once_freeswitch_linger;
      this.once("freeswitch_disconnect_notice", (res) => {
        this.logger.debug("FreeSwitchResponse: auto_cleanup: Received ESL disconnection notice", {
          ref: this.__ref,
          res
        });
        switch (res.headers["Content-Disposition"]) {
          case "linger":
            this.logger.debug("FreeSwitchResponse: Sending freeswitch_linger", {
              ref: this.__ref
            });
            this.emit("freeswitch_linger");
            break;
          case "disconnect":
            this.logger.debug("FreeSwitchResponse: Sending freeswitch_disconnect", {
              ref: this.__ref
            });
            this.emit("freeswitch_disconnect");
            break;
          default:
            this.logger.debug("FreeSwitchResponse: Sending freeswitch_disconnect", {
              ref: this.__ref
            });
            this.emit("freeswitch_disconnect");
        }
      });
      linger_delay = 4e3;
      once_freeswitch_linger = () => {
        this.logger.debug("FreeSwitchResponse: auto_cleanup/linger", {
          ref: this.__ref
        });
        if (this.emit("cleanup_linger")) {
          this.logger.debug("FreeSwitchResponse: auto_cleanup/linger: cleanup_linger processed, make sure you call exit()", {
            ref: this.__ref
          });
        } else {
          this.logger.debug(`FreeSwitchResponse: auto_cleanup/linger: exit() in ${linger_delay}ms`, {
            ref: this.__ref
          });
          setTimeout(() => {
            this.logger.debug("FreeSwitchResponse: auto_cleanup/linger: exit()", {
              ref: this.__ref
            });
            this.exit().catch(function() {
              return true;
            });
          }, linger_delay);
        }
      };
      this.once("freeswitch_linger", once_freeswitch_linger);
      once_freeswitch_disconnect = () => {
        this.logger.debug("FreeSwitchResponse: auto_cleanup/disconnect", {
          ref: this.__ref
        });
        if (this.emit("cleanup_disconnect")) {
          this.logger.debug("FreeSwitchResponse: auto_cleanup/disconnect: cleanup_disconnect processed, make sure you call end()", {
            ref: this.__ref
          });
        } else {
          setTimeout(() => {
            this.logger.debug("FreeSwitchResponse: auto_cleanup/disconnect: end()", {
              ref: this.__ref
            });
            return this.end();
          }, 100);
        }
      };
      this.once("freeswitch_disconnect", once_freeswitch_disconnect);
      return null;
    }
  }
  ;
  FreeSwitchResponse2.prototype.default_event_timeout = 9 * 3600 * 1e3;
  FreeSwitchResponse2.prototype.default_send_timeout = 10 * 1e3;
  FreeSwitchResponse2.prototype.default_command_timeout = 1 * 1e3;
  return FreeSwitchResponse2;
}.call(void 0);

// src/esl.litcoffee
import assert2 from "node:assert";
var default_password;
var FreeSwitchServer = class FreeSwitchServer2 extends EventEmitter2 {
  /**
   * @params { { all_events?: boolean, my_events?: boolean, logger?: any } options
   */
  constructor(options = {}) {
    var all_events, my_events, ref, ref1, ref2;
    super();
    this.logger = (ref = options.logger) != null ? ref : console;
    assert2("function" === typeof this.logger.debug);
    assert2("function" === typeof this.logger.info);
    assert2("function" === typeof this.logger.error);
    all_events = (ref1 = options.all_events) != null ? ref1 : true;
    my_events = (ref2 = options.my_events) != null ? ref2 : true;
    this.__server = new net.Server({
      noDelay: true,
      keepAlive: true
    });
    this.stats = {
      error: 0n,
      drop: 0n,
      connection: 0n,
      connected: 0n,
      connection_error: 0n
    };
    this.__server.on("error", (exception) => {
      this.stats.error++;
      this.logger.error("FreeSwitchServer: server error", exception);
      this.emit("error", exception);
    });
    this.__server.on("drop", (data) => {
      this.stats.drop++;
      this.logger.error("FreeSwitchServer: server drop", data);
      this.emit("drop", data);
    });
    this.__server.on("connection", async (socket) => {
      var Unique_ID, call, connect_response, data, exception, uuid;
      this.stats.connection++;
      this.logger.debug("FreeSwitchServer received connection");
      try {
        call = new FreeSwitchResponse(socket, this.logger);
        Unique_ID = "Unique-ID";
        connect_response = await call.connect();
        data = connect_response.body;
        uuid = data[Unique_ID];
        this.stats.connected++;
        this.logger.debug("FreeSwitchServer received connection: connected", { uuid });
        if (uuid != null) {
          call.setUUID(uuid);
        }
        if (my_events) {
          await call.filter(Unique_ID, uuid);
        }
        await call.auto_cleanup();
        if (all_events) {
          await call.event_json("ALL");
        } else {
          await call.event_json("CHANNEL_EXECUTE_COMPLETE", "BACKGROUND_JOB");
        }
        this.logger.debug("FreeSwitchServer received connection: sending `connection` event", { uuid });
        this.emit("connection", call, { ...connect_response, data, uuid });
      } catch (error1) {
        exception = error1;
        this.stats.connection_error++;
        this.logger.error("FreeSwitchServer: connection handling error", exception);
        this.emit("error", exception);
      }
    });
    this.logger.info("FreeSwitchServer: Ready to start Event Socket server, use listen() to start.");
    return;
  }
  async listen(options) {
    this.__server.listen(options);
    return await once(this.__server, "listening");
  }
  async close() {
    this.__server.close();
    return await once(this.__server, "close");
  }
  getConnectionCount() {
    return new Promise((resolve, reject) => {
      this.__server.getConnections(function(err, count) {
        if (err) {
          return reject(err);
        } else {
          return resolve(count);
        }
      });
    });
  }
};
default_password = "ClueCon";
var FreeSwitchClient = class FreeSwitchClient2 extends EventEmitter2 {
  constructor(options = {}) {
    var ref;
    super();
    this.logger = (ref = options.logger) != null ? ref : console;
    assert2("function" === typeof this.logger.debug);
    assert2("function" === typeof this.logger.info);
    assert2("function" === typeof this.logger.error);
    this.options = {
      host: "127.0.0.1",
      port: 8021,
      password: default_password,
      ...options
    };
    this.current_call = null;
    this.running = true;
    this.retry = 200;
    this.attempt = 0n;
    this.logger.info("FreeSwitchClient: Ready to start Event Socket client, use connect() to start.");
    return;
  }
  /**
   * @return undefined
   */
  connect() {
    var error, ref, socket;
    if (!this.running) {
      this.logger.debug("FreeSwitchClient::connect: not running, aborting", { options: this.options, attempt: this.attempt });
      return;
    }
    this.attempt++;
    this.logger.debug("FreeSwitchClient::connect", { options: this.options, attempt: this.attempt, retry: this.retry });
    if ((ref = this.current_call) != null) {
      ref.end();
    }
    socket = new net.Socket();
    this.current_call = new FreeSwitchResponse(socket, this.logger);
    socket.once("connect", async () => {
      var error2, ref1, ref2, ref3, ref4;
      try {
        await ((ref1 = this.current_call) != null ? ref1.onceAsync("freeswitch_auth_request", 2e4, "FreeSwitchClient expected authentication request") : void 0);
        await ((ref2 = this.current_call) != null ? ref2.auth(this.options.password) : void 0);
        await ((ref3 = this.current_call) != null ? ref3.auto_cleanup() : void 0);
        await ((ref4 = this.current_call) != null ? ref4.event_json("CHANNEL_EXECUTE_COMPLETE", "BACKGROUND_JOB") : void 0);
      } catch (error1) {
        error2 = error1;
        this.logger.error("FreeSwitchClient: connect error", error2);
        this.emit("error", error2);
      }
      if (this.running && this.current_call) {
        this.emit("connect", this.current_call);
      }
    });
    socket.once("error", (error2) => {
      var code;
      code = "code" in error2 ? error2.code : void 0;
      if (this.retry < 5e3) {
        if (code === "ECONNREFUSED") {
          this.retry = Math.floor(this.retry * 1200 / 1e3);
        }
      }
      this.logger.error("FreeSwitchClient::connect: client received `error` event", { attempt: this.attempt, retry: this.retry, error: error2, code });
      if (this.running) {
        this.emit("reconnecting", this.retry);
        setTimeout(() => {
          return this.connect();
        }, this.retry);
      }
    });
    socket.once("end", () => {
      this.logger.debug("FreeSwitchClient::connect: client received `end` event (remote end sent a FIN packet)", { attempt: this.attempt, retry: this.retry });
      if (this.running) {
        this.emit("reconnecting", this.retry);
        setTimeout(() => {
          return this.connect();
        }, this.retry);
      }
    });
    socket.on("warning", (data) => {
      this.emit("warning", data);
    });
    try {
      this.logger.debug("FreeSwitchClient::connect: socket.connect", { options: this.options, attempt: this.attempt, retry: this.retry });
      socket.connect(this.options);
    } catch (error1) {
      error = error1;
      this.logger.error("FreeSwitchClient::connect: socket.connect", { error });
    }
  }
  async end() {
    this.logger.debug("FreeSwitchClient::end: end requested by application.", { attempt: this.attempt });
    this.emit("end");
    this.running = false;
    if (this.current_call != null) {
      await this.current_call.end();
      this.current_call = null;
    }
  }
};
export {
  FreeSwitchClient,
  FreeSwitchResponse,
  FreeSwitchServer
};
